import type { Plugin } from 'vite'
import fs from 'fs'
import prettier from 'prettier'

// uni-app
import uni from '@dcloudio/vite-plugin-uni'

function generateUniComponentsTypes() {
    const tags = fs.readFileSync('./node_modules/@dcloudio/uni-helper-json/dist/tags.json', 'utf8')
    const json = JSON.parse(tags)
    const components = Object.keys(json).map((key) => `'${key}': typeof HTMLElement`)
    const content = `
        // generated by @dcloudio/uni-helper-json/dist/tags.json

        declare module 'vue' {
            // uni-app 内置组件高亮
            export interface GlobalComponents {
                ${components.join(',\n\t')}
            }
        }
        export {}
    `

    const formatted = prettier.format(content, { parser: 'typescript', tabWidth: 4, semi: false })
    fs.writeFileSync('./types/uni-components.d.ts', formatted, 'utf8')
}

/**
 * uni-app
 * @returns Plugin[]
 */
export function configUniPlugin(): Plugin[] {
    // 生成 uni-components.d.ts
    generateUniComponentsTypes()
    return uni()
}
